from enum import Enum


BASE_URL = "https://api.us-east-1.relevance.ai/latest"

WELCOME_MESSAGE = "Welcome to RelevanceAI AI Dashboard. Logged in as {}."

ONE_MB = 2**20


class Colours(Enum):
    INDIGO = "INDIGO"
    TEAL = "TEAL"
    PINK = "PINK"
    VIOLET = "VIOLET"
    YELLOW = "YELLOW"
    BLUE = "BLUE"
